@@ -15,10 +15,9 @@ use meio::{
     InterruptedBy, LiteTask, Scheduled, StartedBy, StopReceiver, TaskEliminated, TaskError,
 };
 use meio_protocol::Protocol;
-use serde::de::DeserializeOwned;
+use serde::{de::DeserializeOwned, Deserialize};
 use slab::Slab;
 use std::future::Future;
-use std::marker::PhantomData;
 use std::net::SocketAddr;
 use std::pin::Pin;
 use std::sync::Arc;
@@ -29,6 +28,10 @@ use tokio::sync::RwLock;
 use tokio_tungstenite::WebSocketStream;
 use tungstenite::protocol::Role;
 
+/// Empty parameters.
+#[derive(Default, Deserialize)]
+pub struct NoParameters {}
+
 #[derive(Debug, Error)]
 #[error("route error [path = {path}, query = {query}]: {reason}")]
 pub struct RouteError {
@@ -48,6 +51,7 @@ impl RouteError {
 }
 
 pub trait DirectPath: Sized + Send + Sync + 'static {
+    type Output: DeserializeOwned + Send;
     type Parameter;
     fn paths() -> &'static [&'static str];
 }
@@ -55,11 +59,10 @@ pub trait DirectPath: Sized + Send + Sync + 'static {
 impl<T> FromRequest for T
 where
     T: DirectPath,
-    Self: DeserializeOwned,
 {
-    type Output = Self;
+    type Output = <T as DirectPath>::Output;
 
-    fn from_request(request: &Request<Body>) -> Result<Option<Self::Output>, Error> {
+    fn from_request(&self, request: &Request<Body>) -> Result<Option<Self::Output>, Error> {
         let uri = request.uri();
         let path = uri.path();
         if Self::paths().iter().any(|p| p == &path) {
@@ -79,12 +82,11 @@ impl<T> WsFromRequest for T
 where
     T: DirectPath,
     T::Parameter: Protocol,
-    Self: DeserializeOwned,
 {
-    type Output = Self;
+    type Output = <T as DirectPath>::Output;
     type Protocol = T::Parameter;
 
-    fn from_request(request: &Request<Body>) -> Result<Option<Self::Output>, Error> {
+    fn from_request(&self, request: &Request<Body>) -> Result<Option<Self::Output>, Error> {
         let uri = request.uri();
         let path = uri.path();
         if Self::paths().iter().any(|p| p == &path) {
@@ -101,7 +103,7 @@ where
 pub trait FromRequest: Sized + Send + Sync + 'static {
     type Output: Send;
 
-    fn from_request(request: &Request<Body>) -> Result<Option<Self::Output>, Error>;
+    fn from_request(&self, request: &Request<Body>) -> Result<Option<Self::Output>, Error>;
 }
 
 pub struct Req<T: FromRequest> {
@@ -124,7 +126,7 @@ pub struct WebRoute<E, A>
 where
     A: Actor,
 {
-    extracted: PhantomData<E>,
+    extractor: E,
     address: Address<A>,
 }
 
@@ -132,11 +134,8 @@ impl<E, A> WebRoute<E, A>
 where
     A: Actor,
 {
-    pub fn new(address: Address<A>) -> Self {
-        Self {
-            extracted: PhantomData,
-            address,
-        }
+    pub fn new(extractor: E, address: Address<A>) -> Self {
+        Self { extractor, address }
     }
 }
 
@@ -149,7 +148,7 @@ where
     A: Actor + ActionHandler<Interact<Req<E>>>,
 {
     fn try_route(&self, _addr: &SocketAddr, request: Request<Body>) -> RouteResult {
-        match E::from_request(&request) {
+        match self.extractor.from_request(&request) {
             Ok(Some(value)) => {
                 let msg = Req {
                     request: value,
@@ -171,7 +170,7 @@ pub trait WsFromRequest: Sized + Send + Sync + 'static {
     type Output: Send;
     type Protocol: Protocol;
 
-    fn from_request(request: &Request<Body>) -> Result<Option<Self::Output>, Error>;
+    fn from_request(&self, request: &Request<Body>) -> Result<Option<Self::Output>, Error>;
 }
 
 pub struct WsReq<T: WsFromRequest> {
@@ -185,7 +184,7 @@ pub struct WsRoute<E, A>
 where
     A: Actor,
 {
-    extracted: PhantomData<E>,
+    extractor: E,
     address: Address<A>,
 }
 
@@ -193,11 +192,8 @@ impl<E, A> WsRoute<E, A>
 where
     A: Actor,
 {
-    pub fn new(address: Address<A>) -> Self {
-        Self {
-            extracted: PhantomData,
-            address,
-        }
+    pub fn new(extractor: E, address: Address<A>) -> Self {
+        Self { extractor, address }
     }
 }
 
@@ -207,7 +203,7 @@ where
     A: Actor + ActionHandler<WsReq<E>>,
 {
     fn try_route(&self, addr: &SocketAddr, mut request: Request<Body>) -> RouteResult {
-        match E::from_request(&request) {
+        match self.extractor.from_request(&request) {
             Ok(Some(value)) => {
                 let mut res = Response::new(Body::empty());
                 let mut address = self.address.clone();
